<?PHP
/*
 * Name: Inflector
 * Author: Michal Tatarynowicz (pies /at/ sputnik /dot/ pl)
 * Version: 0.2.0
 * Release date: 2005-04-06
 * Licence: Public Domain
 *
 * Description:
 * I'm trying to port RoR Inflector class here.
 * Inflector pluralizes and singularizes english nouns.
 * Test with $i = new Inflector(); $i->test();
*/

class Inflector {
	
	function pluralize ($word) {
		$plural_rules = array(
			'/(x|ch|ss|sh)$/'			=> '\1es',       # search, switch, fix, box, process, address
			'/series$/'					=> '\1series',
			'/([^aeiouy]|qu)ies$/'	=> '\1y',
			'/([^aeiouy]|qu)y$/'		=> '\1ies',      # query, ability, agency
			'/(?:([^f])fe|([lr])f)$/' => '\1\2ves', # half, safe, wife
			'/sis$/'						=> 'ses',        # basis, diagnosis
			'/([ti])um$/'				=> '\1a',        # datum, medium
			'/person$/'					=> 'people',     # person, salesperson
			'/man$/'						=> 'men',        # man, woman, spokesman
			'/child$/'					=> 'children',   # child
			'/s$/'						=> 's',          # no change (compatibility)
			'/$/'							=> 's'
		);
		foreach ($plural_rules as $rule => $replacement) {
			if (preg_match($rule, $word)) {
				return preg_replace($rule, $replacement, $word);
			}
		}
	}
	function singularize ($word) {
		$singular_rules = array(
			'/(x|ch|ss)es$/'		   => '\1',
			'/movies$/'				   => 'movie',
			'/series$/'				   => 'series',
			'/([^aeiouy]|qu)ies$/'  => '\1y',
			'/([lr])ves$/'			   => '\1f',
			'/([^f])ves$/'			   => '\1fe',
			'/(analy|ba|diagno|parenthe|progno|synop|the)ses$/' => '\1sis',
			'/([ti])a$/'				=> '\1um',
			'/people$/'					=> 'person',
			'/men$/'						=> 'man',
			'/status$/'					=> 'status',
			'/children$/'				=> 'child',
			'/news$/'					=> 'news',
			'/s$/'						=> ''
		);
		foreach ($singular_rules as $rule => $replacement) {
			if (preg_match($rule, $word)) {
				return preg_replace($rule, $replacement, $word);
			}
		}
	}

	function test () {
		$singulars = array(
			'search', 'switch', 'fix', 'box', 'process', 'address', 'query', 'ability', 
			'agency', 'half', 'safe', 'wife', 'basis', 'diagnosis', 'datum', 'medium', 
			'person', 'salesperson', 'man', 'woman', 'spokesman', 'child', 'page', 'robot');
		$plurals = array(
			'searches', 'switches', 'fixes', 'boxes', 'processes', 'addresses', 'queries', 'abilities', 
			'agencies', 'halves', 'saves', 'wives', 'bases', 'diagnoses', 'data', 'media', 
			'people', 'salespeople', 'men', 'women', 'spokesmen', 'children', 'pages', 'robots');

		$pluralize_errors = 0;
		$singularize_errors = 0;
		foreach (array_combine($singulars, $plurals) as $singular => $plural) {
			if ($this->pluralize($singular) != $plural) {
				debug ("Inflector test {$singular} yelded ".$this->pluralize($singular)." (expected {$plural})");
				$pluralize_errors++;
			}
			if ($this->singularize($plural) != $singular) {
				debug ("Inflector test {$plural} yelded ".$this->singularize($plural)." (expected {$singular})");
				$singularize_errors++;
			}
		}
		
		if ($total_errors = $pluralize_errors + $singularize_errors) {
			debug ("Inflector test ERRORS: {$total_errors} (singularize: {$singularize_errors}, pluralize: {$pluralize_errors})");
		}
		else {
			debug ("Inflector test OK.");
		}
	}
}

if (!function_exists('array_combine')) {
	function array_combine($a1, $a2) {
		$a1 = array_values($a1);
		$a2 = array_values($a2);

		if (count($a1) != count($a2)) return false; // different lenghts
		if (count($a1) <= 0) return false; // arrays are the same and both are empty
		
		$output = array();
		
		for ($i = 0; $i < count($a1); $i++) {
			$output[$a1[$i]] = $a2[$i];
		}
		
		return $output;
	}
}

?>