<?PHP
/*
 * Name: Controller
 * Author: Michal Tatarynowicz (pies /at/ sputnik /dot/ pl)
 * Version: 0.2.0
 * Release date: 2005-04-06
 * Licence: Public Domain
 *
 * Description:
 * Application controller (controllers are where you put all the actual code) based on RoR (www.rubyonrails.com)
 * Provides basic functionality, such as rendering views (aka displaying templates).
 * Automatically selects model name from on singularized object class name 
 * and creates the model object if proper class exists.
*/

class Controller {
	// public
	var $layout = 'default';
	var $admin = 0;
	// private
	var $_view_vars = array();
	var $_page_title = false;
	var $_crumbs = array();

	function Controller () {
		global $DB;

		$model_name = Inflector::singularize(get_class($this));

		if (class_exists($model_name))
			$this->$model_name = new $model_name ();
	}

	function set($one, $two=null) {
		return $this->_set_array(is_array($one)? $one: array($one=>$two));
	}
	function set_title ($value) {
		$this->_page_title = $value;
	}
	function _set_array($data) {
		foreach ($data AS $name => $value)
			$this->_view_vars[$name] = $value;;
	}
	function set_layout ($layout) {
		$this->layout = $layout;
	}

	function render ($action, $layout=null) {
		if ($layout) $this->set_layout($layout);

		$view_fn = $this->_get_view_fn($action);

		if (is_file($view_fn)) {
			$out = $this->_do_render($view_fn, $this->_view_vars);

			if ($out === false) {
				$out = $this->_do_render($view_fn, $this->_view_vars, false);
				trigger_error (sprintf(ERROR_IN_VIEW, $view_fn, $out), E_USER_ERROR);
			}
			else {
				print ($this->render_layout($out).($this->admin? $this->admin_menu (): null));
			}
		}
		else {
			trigger_error (sprintf(ERROR_NO_VIEW, $action, $view_fn), E_USER_ERROR);
		}

	}
	function render_layout ($content_for_layout) {
		$layout_fn = $this->_get_layout_fn();

		$data_for_layout = array_merge($this->_view_vars, array(
			'title_for_layout'=>$this->_page_title !== false? $this->_page_title: ucfirst(get_class($this)),
			'content_for_layout'=>$content_for_layout));

		if (is_file($layout_fn)) {
			$out = $this->_do_render($layout_fn, $data_for_layout);

			if ($out === false) {
				$out = $this->_do_render($view_fn, $data_for_layout, false);
				trigger_error (sprintf(ERROR_IN_LAYOUT, $layout_fn, $out), E_USER_ERROR);
			}
			else {
				return $out;
			}
		}
		else {
			trigger_error (sprintf(ERROR_NO_LAYOUT, $this->layout, $layout_fn), E_USER_ERROR);
		}
	}
	function _get_layout_fn() {
		return VIEWS_DIR."layouts/{$this->layout}.thtml";
	}
	function _get_view_fn($action) {
		return VIEWS_DIR.get_class($this)."/{$action}.thtml";
	}
	function _do_render($___view_fn, $___data_for_view, $___play_safe = true) {
		extract($___data_for_view, EXTR_SKIP); # load all view variables
		$URL = $this->params['url'];
		ob_start(); # start caching output (eval outputs directly so we need to cache)
		
		# include the template
		$___play_safe? @include($___view_fn): include($___view_fn);

		$out = ob_get_contents(); # retrieve cached output
		ob_end_clean(); # end caching output

		return $success !== false? $out: ($___play_safe? false: $out); # figure it out yourself :P
	}

	function redirect ($url) {
		header ('Location: '.$this->base.$url);
	}

	function link_for ($title, $url, $confirm=false) {
		return 
		"<a href=\"{$url}\""
		.($confirm? ' onClick="return confirm(\''.$confirm.'\')"': null)
		.'>'
		.$title
		.'</a>';
	}
	function admin_link_for ($title, $url, $confirm=false, $style=null) {
		return $this->admin?
		"<a class=\"admin-link\" href=\"{$url}\""
		.($confirm? ' onClick="return confirm(\''.$confirm.'\')"': null)
		.($style? ' style="'.$style.'"': null)
		.'>'
		.$title
		.'</a>': null;
	}

	function form_tag ($target, $type='post') {
		return '<form class="admin" action="'.$target.'" method="'
		.($type == 'get'? 'get': 'post')
		.'"'
		.($type == 'multipart'? ' enctype="multipart/form-data"': null)
		.'>';
	}
	function form_input_field ($name, $size=20, $value=null) {
		return '<input name="'.$name.'" size="'.$size.'" value="'.$value.'" />';
	}
	function form_file_field ($name) {
		return '<input name="'.$name.'" type="file" />';
	}
	function form_submit ($caption) {
		return '<input type="submit" value="'.$caption.'" />';
	}

	function table_headers () {
		$args = func_get_args();
		$out = false;
		foreach ($args as $arg)
			$out .= "<th>{$arg}</th> ";
		return $out;
	}
	function put_image ($url, $alt=null, $width=null, $height=null) {
		return 
		"<img src=\"{$url}\""
		.($alt? " alt=\"{$alt}\"": ($alt === false? null: ' alt=""'))
		.($width? " width=\"{$width}\"": null)
		.($height? " height=\"{$height}\"": null)
		.' />';
	}
	function _parse_func_args($args) {
		$out = false;
		foreach ($args as $arg) {
			if (ereg('([a-zA-Z][a-zA-Z0-9]*)=([a-zA-Z0-9]*)', $arg, $regs))
				$out[$regs[1]] = $regs[2];
		}
		return $out;
	}

	function add_crumb ($name, $link) {
		$this->_crumbs[] = array ($name, $link);
	}
	function get_crumbs () {

		if (count($this->_crumbs)) {

			foreach ($this->_crumbs as $crumb) {
				$out[] = "<a href=\"{$this->base}{$crumb[1]}\">{$crumb[0]}</a>";
			}
		
			return '<b>Jesteś tutaj:</b> &raquo; '.join(' &raquo; ', $out);
		}
		else
			return null;
	}
}

?>