<?PHP
/*
 * Name: DBO
 * Author: Michal Tatarynowicz (pies /at/ sputnik /dot/ pl)
 * Version: 1.3.0
 * Release date: 2005-03-31
 * Licence: Public Domain
 *
 * Description: 
 * A MySQL functions wrapper. Provides ability to get results as arrays
 * and query logging (with execution time).
 *
 * Example usage:
 *
	require_once('io_dbo.lib');

	// create and connect the object
	$db = new DBO(array(
		'host'=>'localhost',
		'login'=>'username',
		'password'=>'password',
		'database'=>'database'));

	// read the whole query result array (of rows)
	$all_rows = $db->all("SELECT a,b,c FROM table");

	// read the first row with debugging on
	$first_row_only = $db->one("SELECT a,b,c FROM table WHERE a=1", TRUE);

	// emulate the usual MySQL way of reading query results
	if ($db->q("SELECT a,b,c FROM table")) {
		while ( $row = $db->farr() ) {
			print $row['a'].$row['b'].$row['c'];
		}
	}
	
	// show a log of all queries, sorted by execution time
	$db->show_log(TRUE);

*/

class DBO {
	
	// public
	var $connected=FALSE;
	var $debug=FALSE;
	var $error=NULL;
	var $insert_id=NULL;
	var $affected=NULL;
	var $took=NULL;

	// private
	var $_conn=NULL;
	var $_result=NULL;
	var $_queries_cnt=0;
	var $_queries_time=NULL;
	var $_queries_log=array();

	function DBO($config=NULL,$DEBUG=FALSE) { 	
		$this->__constructor($config,$DEBUG); 
	}
	function __constructor($config=NULL,$DEBUG=FALSE) {
		$this->debug = $DEBUG;
		if ($DEBUG > 1) register_shutdown_function( array( &$this, "show_log" ) );
		Return $this->connect($config);
	}
	function __destructor() {
		$this->close();
	}

	function connect($config) {
		if($config) {
			$this->config = $config;
			$this->_conn = @mysql_connect($config['host'],$config['login'],$config['password']);
		}
		$this->connected = $this->_conn? TRUE: FALSE;

		if($this->connected)
			Return @mysql_select_db($config['database'], $this->_conn);
		else
			die('Could not connect to DB.');
	}
	function close() {
		@mysql_close();
		showLog();
		$this->_conn = NULL;
		$this->connected = NULL;
	}

	function q($q,$DEBUG=FALSE,$log=TRUE) {
		Return $this->query($q,$DEBUG,$log);
	}
	function query($q,$DEBUG=FALSE,$log=TRUE) {
		$t = getMicrotime();

		if($log){
			$this->_result = @mysql_query($q);
			$this->took = round((getmicrotime()-$t)*1000, 0);
			$this->error = mysql_errno()? mysql_errno().': '.mysql_error(): NULL;
			$this->insert_id = @mysql_insert_id();
			$this->affected = @mysql_affected_rows();
			$this->num_rows = @mysql_num_rows($this->_result);
			$this->_log_query($q);

			if($this->debug || $DEBUG) $this->_show_query($q);

			Return $this->error? FALSE: $this->_result;
		}
		else
			Return @mysql_query($q);
	}
	function farr($results,$type=MYSQL_BOTH) {
		return mysql_fetch_array($results,$type);
	}
	function one($q,$DEBUG=FALSE,$type=MYSQL_BOTH) {
		Return $this->query($q,$DEBUG)? mysql_fetch_array($this->_result, $type): FALSE;
	}
	function all($q,$DEBUG=FALSE) {
		if($this->query($q,$DEBUG)) {
			$out=NULL;
			while($item = mysql_fetch_assoc($this->_result)) $out[] = $item;

			Return $out;
		}
		else {
			Return FALSE;
		}
	}
	function field($name, $q, $DEBUG=FALSE, $type=MYSQL_BOTH) {
		$data = $this->one($q, $DEBUG, $type);
		return empty($data[$name])? false: $data[$name];
	}

	function tables() {
		$result = mysql_list_tables($this->config['database']);

		if (!$result) {
			trigger_error(ERROR_NO_TABLE_LIST, E_USER_NOTICE);
			exit;
		}
		else {
			$tables = array();
			while ($line = mysql_fetch_array($result)) {
				$tables[] = $line[0];
			}
			return $tables;
		}
	}
	function has_any($table, $sql) {
		$out = $this->one("SELECT COUNT(*) AS count FROM {$table} WHERE {$sql}");
		return is_array($out)? $out['count']: FALSE;
	}

	function got_connected() {
		Return $this->connected;
	}
	function got_insert_id() {
		Return $this->insert_id;
	}
	function got_affected() {
		Return $this->affected;
	}
	function got_num_rows() {
		Return $this->num_rows;
	}
	function got_error() {
		return $this->error;
	}

	function show_log($sorted=FALSE) {
		$log = $sorted?
			sortByKey($this->_queries_log, 'took', 'desc', SORT_NUMERIC):
			$this->_queries_log;

		print("<table border=1>\n<tr><th colspan=7>{$this->_queries_cnt} queries took {$this->_queries_time} s</th></tr>\n");
		print("<tr><td>Nr</td><td>Query</td><td>Error</td><td>Affected</td><td>Num. rows</td><td>Took (ms)</td></tr>\n");

		foreach($log AS $k=>$i) {
			print("<tr><td>".($k+1)."</td><td>{$i['query']}</td><td>{$i['error']}</td><td align='right'>{$i['affected']}</td><td align='right'>{$i['num_rows']}</td><td align='right'>{$i['took']}</td></tr>\n");
		}

		print("</table>\n");
	}
	function _log_query($q, $error=NULL) {
		$this->_queries_cnt++;
		$this->_queries_time += $this->took;
		$this->_queries_log[] = array(
			'query'=>$q,
			'error'=>$this->error,
			'affected'=>$this->affected,
			'num_rows'=>$this->num_rows,
			'took'=>$this->took
			);


		if ($this->error && function_exists('logError'))
			logError("Query: {$q} RETURNS ERROR {$this->error}");
	}
	function _show_query($q) {
		global $_DEBUG;

		$error = $this->error;

		if ($this->debug || $_DEBUG || $error) {
			print("<p style=\"text-align:left\"><b>Query:</b> {$q} <small>[Aff:{$this->affected} Num:{$this->num_rows} Took:{$this->took}ms]</small>");
			if($error) {
				print("<br /><span style=\"color:Red;text-align:left\"><b>ERROR:</b> {$this->error}</span>");
			}
			print('</p>');
		}
	}

}

if (!function_exists('getMicrotime')) {
	function getMicrotime() {
		list($usec, $sec) = explode(" ", microtime());
		return ((float)$usec + (float)$sec);
	}
}
if (!function_exists('sortByKey')) {
	function sortByKey(&$array, $sortby, $order='asc', $type=SORT_NUMERIC) {

		if( is_array($array) ) {

			foreach( $array AS $key => $val )
				$sa[$key] = $val[$sortby];

			if( $order == 'asc' )
				asort($sa, $type);
			else
				arsort($sa, $type);

			foreach( $sa as $key=>$val )
				$out[] = $array[$key];

			Return $out;

		}
		else
			Return null;
	}
}

?>